
$fs = 0.2;
$fa = $fs;



// Material definitions
GreenPLA = [28/255, 1.0, 47/255, 132/255];
BlackAluminium = [16/255, 16/255, 16/255];




// hs = heat sink
hs_l = 120;
hs_w = 79;
hs_baseplate = 4;

hs_fin_w0 = 2;
hs_fin_w1 = 1.3;
hs_fin_d = 11;  // depth
hs_fin_cnt = 10;
hs_fin_spacing = 3.5;

/**
  *
  * TODO: Moar documentation!
  *
  */
module symmetricHeatsink(
  l = hs_l,
  w = hs_w,
  baseplate = hs_baseplate,
  fin_w0 = hs_fin_w0,
  fin_w1 = hs_fin_w1,
  fin_d = hs_fin_d,
  fin_cnt = hs_fin_cnt,
  fin_spacing = hs_fin_spacing
) {
  
  module fin() {
    dw2 = fin_d - fin_w1/2;

    translate([0, fin_w0, dw2]) rotate([-90, 0, -90]) linear_extrude(l) hull() {
      translate([0, dw2, 0]) square([fin_w0, 0.01]);
      translate([fin_w1/2, 0, 0]) circle(d=fin_w1);
    }
  }
  
  module half() {
    cube([l, w/2, baseplate]);

    for (y=[0:fin_cnt/2 - 1]) translate([0, y * (  fin_w0 + fin_spacing), baseplate]) fin();
  }
  
  color(BlackAluminium) translate([-l/2, -w/2, 0]) {
    half();
    translate([0, w, 0]) mirror([0, 1, 0]) half();
  }
}


// f = fan
f_r = 4;  // corner radius of fan case
f_d = 20;  // depth. or thickness.
f_wh = 40;  // widht and height of the fan's chassis
f_drill = 4;  // mounting hole diameter
f_drill_wh = 32;  // size of a virtual rectangle with mounting holes drilled through the corners
f_intake = 38;  // air intake diameter
f_motor = 23;  // diameter of fan motor


/**
  *
  * TODO: Moar documentation!!!
  * 
  * `layer`: either "Body" or "MotorConnection". the latter produces the bridges to the motor
  *
  */
module fanProfile(
  r = f_r,
  wh = f_wh,
  drill = f_drill,
  drill_wh = f_drill_wh,
  intake = f_intake,
  motor = f_motor,

  layer = "Body"
) {
  
  difference() {
    roundRect(wh, wh, r);
    
    // mounting holes
    _dwh2 = drill_wh / 2;
    for (tx=[-_dwh2, _dwh2], ty=[-_dwh2, _dwh2]) translate([tx, ty, 0]) circle(d=drill);
    
    // the central air intake
    circle(d=intake);
  }

  // the fans motor
  if (motor > 0) circle(d=motor);
}


/**
  *
  * TODO: Moar documentation!!!
  *
  */
module fan(
  d = f_d,
  r = f_r,
  wh = f_wh,
  drill = f_drill,
  drill_wh = f_drill_wh,
  intake = f_intake,
  motor = f_motor,
  layer_ratio = 0.1
) {

  // shadow original module here so we don't have to repeat the param list ^^
  module _fanProfile(layer) {
    fanProfile(r=r, wh=wh, drill=drill, drill_wh=drill_wh, intake=intake, motor=motor, layer=layer);
  }

  if (layer_ratio > 0) translate([0, 0, d - d * layer_ratio]) linear_extrude(d * layer_ratio) _fanProfile("MotorConnection");

  if (layer_ratio < 1) linear_extrude(d * (1 - layer_ratio)) _fanProfile("Body");  
}



fu_strength = 2;  // material strength

fu_outlet_w = (hs_fin_cnt / 2) * hs_fin_w0 + (hs_fin_cnt / 2 - 1) * hs_fin_spacing;  // Σ of fins spacing and width calculated over one side;







module tri(w, h) {
  polygon([
    [0, -h/2],
    [0, h/2],
    [w, 0]
  ]);
}


// length of the outlet
x = 5;  // TODO: rename!!!!

// The profile of the funnel, ready for extrusion
module funnel2D() {
  
  _y = hs_w/2 - fu_outlet_w;

  difference() {
    
    union() {
      translate([0, -hs_w/2, 0]) square([x, hs_w]);
      translate([0, 0, 0]) roundTri(x*9, hs_w, 5);
    }
    
    /* union() {
      translate([0, _y, 0]) square([x, fu_outlet_w]);
      translate([0, -_y - fu_outlet_w, 0]) square([x, fu_outlet_w]);
      
      // air funnel split
      translate([x, 0, 0]) tri(_y*4, hs_w);
    } */

    // air funnel split
    translate([0, 0, 0]) roundTri(x*4, _y*2, 3);
    translate([0, -_y, 0]) square([x/2, _y*2]);
  }
}



// ff = funnel fin
ff_str = hs_fin_w0;  // strength

module funnelFins(
  w = hs_w,
  fin_cnt = hs_fin_cnt,
  fin_spacing = hs_fin_spacing,
  fin_cnt = hs_fin_cnt,
  fin_w0 = hs_fin_w0,
  _str = ff_str
) {
  
  module fin(x, y) {
    module _fF(y) {
      translate([0, -y/2, 0]) square([x, y]);
      translate([0, 0, 0]) roundTri((x*9) * (y/w), y, 5);
    }

    difference() {
      _fF(y + _str/2);
      _fF(y - _str/2);
    }
  }
  
  _x = 5;
  for (y=[0:fin_cnt/2 - 1]) fin( -(y/fin_cnt/2 - 1) * _x,  w - y * (fin_w0 + fin_spacing) * 2);
}





module  funnel() {
  color(GreenPLA) {
    
    // TODO: implement!
    
    linear_extrude(1) funnel2D();
  }
}




// ############################
// ####### New Approach #######
// ############################


// TODO: link nad include the WebWeaver and write the code ^^





module preview() {

  // funnel();
  // color([0, 0, 1.0, 1.0]) translate([0, 0, 1]) linear_extrude(1) funnelFins();
  
  // rotate([180, 0, 0]) symmetricHeatsink();
}


preview();

